package main;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import mold.SD_PBSTools_HelditemNames;
import mold.SD_PBSTools_MoveNames;
import mold.SD_PBSTools_SpeciesNames;
import party.PartyInfo;

/*
*  Sample10_PBSToolsPartyToSDTeamForNC2000クラス
*
*  サンプルのメインクラスです。
*  PBSToolsのパーティテキスト（複数）を、Pokemon Showdown!のTeamBuilderのテキストに変換します。
*  ニンテンドウカップ２０００に対応し、ポケスタ環境準拠かGB・VC環境準拠かは対話式で指定します。
*  （一応、Gen2OUにも対応しています）
*
*  変更履歴
*  2021/07/04 新規作成
*  2021/07/11 Pokemon Showdown!向けの努力レベルの算出式を訂正
*
*/

public class Sample10_PBSToolsPartyToSDTeamForNC2000 {

	@SuppressWarnings("null")
	public static void main(String[] args) {

		/* 変数定義 */
		// スキャナー本体
		Scanner scanner = new Scanner(System.in);

		// ルール指定
		int ruleFlag = -1; // 0ならポケスタ環境準拠、1ならGB・VC環境準拠

		// パス指定
		String PBSToolsForderPath = "sub_tools" + "\\" + "005_Conf_to_SD" + "\\" + "Conf";
		String SDForderPath = "sub_tools" + "\\" + "005_Conf_to_SD" + "\\" + "SD";
		String SDFileName = "localStorage_UTF-8.txt";
		String SDFilePath = SDForderPath + "\\" + SDFileName;

		// 書き込みストリーム
		BufferedWriter bw = null;

		// 変換用オブジェクト生成
		SD_PBSTools_SpeciesNames SD_PBSTools_SpeciesNames_Obj = new SD_PBSTools_SpeciesNames();
		SD_PBSTools_MoveNames SD_PBSTools_MoveNames_Obj = new SD_PBSTools_MoveNames();
		SD_PBSTools_HelditemNames SD_PBSTools_HelditemNames_Obj = new SD_PBSTools_HelditemNames();

		/* 処理 */
		// 環境指定
		while (ruleFlag != 0 && ruleFlag != 1) {
			System.out.println("ポケスタ環境準拠なら0、GB・VC環境準拠（もしくはOU）なら1を入力して下さい。");
			ruleFlag = Integer.parseInt(scanner.nextLine());
		}
		scanner.close();

		// 書き込みストリーム生成（追記）
		try {
			bw = new BufferedWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(SDFilePath, true), "UTF-8")));
		} catch (UnsupportedEncodingException e1) {
			System.out.println("ソースの記述ミスで出力ファイルをオープンできないので処理を終了します。");
			try {
				bw.close();
			} catch (Exception e) {
			}
			return;
		} catch (FileNotFoundException e1) {
			System.out.println(SDFilePath + " というファイルパス・ファイル名で空のテキストファイルを用意して下さい。");
			try {
				bw.close();
			} catch (Exception e) {
			}
			return;
		}

		// 各パーティテキストに対して処理
		File PBSToolsForder = new File(PBSToolsForderPath);
        File PBSToolsFiles[] = PBSToolsForder.listFiles();
        for (int i = 0; i < PBSToolsFiles.length; i++) {

        	// 変換オブジェクトのリセット処理
        	SD_PBSTools_HelditemNames_Obj.counterReset();

            // ファイル読み込み
        	System.out.println("---------------------------------------------");
        	System.out.println(PBSToolsFiles[i] + " の変換");
            BufferedReader br = null;
            PartyInfo partyInfo = new PartyInfo();
            try {
            	br = new BufferedReader(new InputStreamReader(new FileInputStream(PBSToolsFiles[i]),"SJIS"));
            	String str = br.readLine();
            	int j = 0;
            	if(str == null) {
            		throw new NullPointerException();
            	} else {
            		while(str != null && j < 6){
            			partyInfo.partyTextArray[j] = str;
            			j++;
            			str = br.readLine();
            		}
            	}
            } catch (Exception e) {
            	System.out.println("パーティテキストファイルを読み込めないため変換に失敗しました。");
            	continue;
            } finally {
            	try {
    				br.close();
    			} catch (IOException e) {
    			}
            }
            partyInfo.partyPokeMoldSet();

            // ファイル書き込み
            try {

            	// ヘッダー部
            	String partyName = PBSToolsFiles[i].toString().replace(PBSToolsForderPath + "\\", "").replace(".txt", "");
            	if (ruleFlag == 0) {
					bw.write("=== [gen2nintendocup2000] " + partyName + " ===");bw.newLine();
            	} else {
            		bw.write("=== [gen2ou] " + partyName + " ===");bw.newLine();
            	}
            	bw.newLine();

            	// データ部
        		for (int j = 0; j < 6; j++) {
        			if (partyInfo.pokeMoldArray[j] != null) {

                    	// ニックネーム・種族名・性別・持ち物名の行
                    	String speciesLine = SD_PBSTools_SpeciesNames_Obj.PBSToolsToSD(partyInfo.pokeMoldArray[j].speciesName) + " ";
                    	if (partyInfo.pokeMoldArray[j].sex.equals("male")) {
                    		speciesLine = speciesLine + "(M) ";
                    	} else if (partyInfo.pokeMoldArray[j].sex.equals("female")) {
                    		speciesLine = speciesLine + "(F) ";
                    	}
                    	if (!partyInfo.pokeMoldArray[j].itemName.equals("持ち物無し")) {
                    		speciesLine = speciesLine + "@ " + SD_PBSTools_HelditemNames_Obj.PBSToolsToSD(partyInfo.pokeMoldArray[j].itemName)  + " ";
                    	}
                    	speciesLine = speciesLine + " ";
                    	bw.write(speciesLine);bw.newLine();

                    	// 特性名の行
                    	bw.write("Ability: none  ");bw.newLine();

                    	// レベルの行
                    	if (partyInfo.pokeMoldArray[j].level < 100) {
                    		bw.write("Level: " + partyInfo.pokeMoldArray[j].level + "  ");bw.newLine();
                    	}

                    	// 懐き度の行
                    	int returnCount = 0;
                    	int frustrationCount = 0;
                    	for (int k = 0; k < 4; k++) {
                    		if (partyInfo.pokeMoldArray[j].moveName[k].equals("おんがえし")) {
                    			returnCount++;
                    		} else if (partyInfo.pokeMoldArray[j].moveName[k].equals("やつあたり")) {
                    			frustrationCount++;
                    		}
                    	}
                    	if (returnCount == 0 && frustrationCount > 1) {
                    		bw.write("Happiness: 0  ");bw.newLine();
                    	}

                    	// 努力レベルの行
                		boolean gen2ouAdjustFlag = false;
                		if (ruleFlag == 1 &&
                			partyInfo.pokeMoldArray[j].level < 51) {
                			gen2ouAdjustFlag = true;
                		}
                    	if (gen2ouAdjustFlag ||
                    		partyInfo.pokeMoldArray[j].HPEffortLevel != 63 ||
                    	    partyInfo.pokeMoldArray[j].attackEffortLevel != 63 ||
                    	    partyInfo.pokeMoldArray[j].blockEffortLevel != 63 ||
                    	    partyInfo.pokeMoldArray[j].speedEffortLevel != 63 ||
                    	    partyInfo.pokeMoldArray[j].specialEffortLevel != 63) {
                    		boolean slashFlag = false;
                    		String effortLevelLine = "EVs: ";
                    		if (gen2ouAdjustFlag) {
                    			if (partyInfo.pokeMoldArray[j].HPEffortLevel == 63) {
                    				effortLevelLine = effortLevelLine + "251 HP ";
                    			} else {
                    				effortLevelLine = effortLevelLine + ((partyInfo.pokeMoldArray[j].HPEffortLevel * 4) + 1) + " HP ";
                    			}
                    			slashFlag = true;
                    		} else if (partyInfo.pokeMoldArray[j].HPEffortLevel != 0) {
                    			effortLevelLine = effortLevelLine + (partyInfo.pokeMoldArray[j].HPEffortLevel * 4) + " HP ";
                    			slashFlag = true;
                    		// EVsが空になるのを回避
                    		} else if (partyInfo.pokeMoldArray[j].HPEffortLevel == 0 ||
                            	        partyInfo.pokeMoldArray[j].attackEffortLevel == 0 ||
                            	        partyInfo.pokeMoldArray[j].blockEffortLevel == 0 ||
                            	        partyInfo.pokeMoldArray[j].speedEffortLevel == 0 ||
                            	        partyInfo.pokeMoldArray[j].specialEffortLevel == 0) {
                    			effortLevelLine = effortLevelLine + "1 HP ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].attackEffortLevel != 0) {
                    			if (slashFlag) {
                    				effortLevelLine = effortLevelLine + "/ ";
                    			}
                    			effortLevelLine = effortLevelLine + (partyInfo.pokeMoldArray[j].attackEffortLevel * 4) + " Atk ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].blockEffortLevel != 0) {
                    			if (slashFlag) {
                    				effortLevelLine = effortLevelLine + "/ ";
                    			}
                    			effortLevelLine = effortLevelLine + (partyInfo.pokeMoldArray[j].blockEffortLevel * 4) + " Def ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].specialEffortLevel != 0) {
                    			if (slashFlag) {
                    				effortLevelLine = effortLevelLine + "/ ";
                    			}
                    			int specialEffortLevelSD = partyInfo.pokeMoldArray[j].specialEffortLevel * 4;
                    			effortLevelLine = effortLevelLine + specialEffortLevelSD + " SpA / " + specialEffortLevelSD + " SpD ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].speedEffortLevel != 0) {
                    			if (slashFlag) {
                    				effortLevelLine = effortLevelLine + "/ ";
                    			}
                    			effortLevelLine = effortLevelLine + (partyInfo.pokeMoldArray[j].speedEffortLevel * 4) + " Spe ";
                    		}
                    		effortLevelLine = effortLevelLine + " ";
                    		bw.write(effortLevelLine);bw.newLine();
                    	}

                    	// 個体値の行
                    	if (partyInfo.pokeMoldArray[j].HPIndividualValue != 15 ||
                    		partyInfo.pokeMoldArray[j].attackIndividualValue != 15 ||
                    		partyInfo.pokeMoldArray[j].blockIndividualValue != 15 ||
                    		partyInfo.pokeMoldArray[j].speedIndividualValue != 15 ||
                    		partyInfo.pokeMoldArray[j].specialIndividualValue != 15) {
                    		boolean slashFlag = false;
                    		String individualValueLine = "IVs: ";
                    		if (partyInfo.pokeMoldArray[j].HPIndividualValue != 15) {
                    			individualValueLine = individualValueLine + (partyInfo.pokeMoldArray[j].HPIndividualValue * 2) + " HP ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].attackIndividualValue != 15) {
                    			if (slashFlag) {
                    				individualValueLine = individualValueLine + "/ ";
                    			}
                    			individualValueLine = individualValueLine + (partyInfo.pokeMoldArray[j].attackIndividualValue * 2) + " Atk ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].blockIndividualValue != 15) {
                    			if (slashFlag) {
                    				individualValueLine = individualValueLine + "/ ";
                    			}
                    			individualValueLine = individualValueLine + (partyInfo.pokeMoldArray[j].blockIndividualValue * 2) + " Def ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].specialIndividualValue != 15) {
                    			if (slashFlag) {
                    				individualValueLine = individualValueLine + "/ ";
                    			}
                    			int specialIndividualValueSD = partyInfo.pokeMoldArray[j].specialIndividualValue * 2;
                    			individualValueLine = individualValueLine + specialIndividualValueSD + " SpA / " + specialIndividualValueSD + " SpD ";
                    			slashFlag = true;
                    		}
                    		if (partyInfo.pokeMoldArray[j].speedIndividualValue != 15) {
                    			if (slashFlag) {
                    				individualValueLine = individualValueLine + "/ ";
                    			}
                    			individualValueLine = individualValueLine + (partyInfo.pokeMoldArray[j].speedIndividualValue * 2) + " Spe ";
                    		}
                    		individualValueLine = individualValueLine + " ";
                    		bw.write(individualValueLine);bw.newLine();
                    	}

                    	// 技の行
                    	for (int k = 0; k < 4; k++) {
                    		if (!partyInfo.pokeMoldArray[j].moveName[k].equals("技無し")) {
                    			String moveNameSD = SD_PBSTools_MoveNames_Obj.PBSToolsToSD(partyInfo.pokeMoldArray[j].moveName[k]);
                    			if (!moveNameSD.equals("No Move")) {
                    				bw.write("- " + moveNameSD + "  ");bw.newLine();
                    			}

                    		}
                    	}

                    	// 区切り行
                    	bw.newLine();
        			}
        		}

        		// トレーラー部
        		bw.newLine();
        		System.out.println("書き込み完了。");

			} catch (IOException e) {
				System.out.println("途中でファイル書き込みに失敗しました。書き込み結果を確認して下さい。");
			}

        }

        // Writerクローズ
        try {
			bw.close();
		} catch (IOException e) {
		}

	}

}
