package main;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import mold.SD_PBSTools_HelditemNames;
import mold.SD_PBSTools_MoveNames;
import mold.SD_PBSTools_SpeciesNames;

/*
*  Sample11_SDTeamToPBSToolsPartyクラス
*
*  サンプルのメインクラスです。
*  Pokemon Showdown!のTeamBuilderのテキストを、PBSToolsのパーティテキスト（複数）に変換します。
*
*  変更履歴
*  2021/07/08 新規作成
*
*/

public class Sample11_SDTeamToPBSToolsParty {

	public static void main(String[] args) {

		/* 変数定義 */
		// パス指定
		String SDForderPath = "sub_tools" + "\\" + "006_SD_to_Conf" + "\\" + "SD";
		String SDFileName = "localStorage_UTF-8.txt";
		String SDFilePath = SDForderPath + "\\" + SDFileName;
		String PBSToolsForderPath = "sub_tools" + "\\" + "006_SD_to_Conf" + "\\" + "Conf";
		String PBSToolsFileName = "";
		File PBSToolsFilePath = null;

		// ファイルストリーム
		BufferedReader br = null;
		BufferedWriter bw = null;

		// 読み込んだ文字列
		String str = null;
		StringBuilder sb = new StringBuilder();

		// ファイル読み込み状態
		//   0…ヘッダー部読込前
		//   1…ヘッダー部読込中
		//   2…データ部読込前
		//   3…データ部読込中
		int state = 0;

		// 読込完了ポケモン匹数
		int pokemonDoneNum = 0;

		// 読込完了技数
		int moveDoneNum = 0;

		// ポケモン情報
		int level = 100; //レベル
		String speciesName = "けつばん"; // 種族名
		String[] moveName = {"-","-","-","-"}; // 技名
		String itemName = "-"; // 持ち物
		String individualValue = "FFFF"; // 個体値
		String effortLevel = "63-63-63-63-63"; // 努力レベル

		// 変換用オブジェクト生成
		SD_PBSTools_SpeciesNames SD_PBSTools_SpeciesNames_Obj = new SD_PBSTools_SpeciesNames();
		SD_PBSTools_HelditemNames SD_PBSTools_HelditemNames_Obj = new SD_PBSTools_HelditemNames();
		SD_PBSTools_MoveNames SD_PBSTools_MoveNames_Obj = new SD_PBSTools_MoveNames();

		/* 処理 */
		try {
			// ファイルオープン
			br = new BufferedReader(new InputStreamReader(new FileInputStream(SDFilePath),"UTF-8"));

			// 先読み
			str = br.readLine();

			// 最後まで繰り返し読む
			while (str != null) {

				// ヘッダー部読込前
				if (state == 0) {

					// ヘッダー部でなければ無視
					if (!str.substring(0,5).equals("=== [")) {
						continue;
					}

					// 書込ファイルストリーム初期化
					bw = null;

					// パーティ名読込
					sb.delete(0, sb.length());
					int subState = 0;
					for(int i = 0; i < str.length(); i++) {
						if (subState == 0 && String.valueOf(str.charAt(i)).equals("]")) {
							subState = 1;
							i++;
						} else if (subState == 1){
							sb.append(str.charAt(i));
						}
					}
					PBSToolsFileName = sb.toString().substring(0, sb.length() - 4);
					PBSToolsFilePath = new File(PBSToolsForderPath + "\\" + PBSToolsFileName + ".txt");
			        while (PBSToolsFilePath.exists()) {
			        	Calendar calender = Calendar.getInstance();
			            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssSSS");
			            String dateTime = sdf.format(calender.getTime());
			            PBSToolsFilePath = new File(PBSToolsForderPath + "\\" + PBSToolsFileName + "_" + dateTime + ".txt");
			        }

					// 書込ファイルストリームオープン
					bw = new BufferedWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(PBSToolsFilePath), "SJIS")));
					System.out.println(PBSToolsFileName + " の書き込みを開始します。");

					// ヘッダー部読込中へ
					state = 1;

				// ヘッダー部読込中
				} else if (state == 1) {

					// 空白判定（ヘッダー部終了）
					if (str.length() == 0) {

						// データ部読込前へ
						state = 2;

					}

				// データ部読込前
				} else if (state == 2) {

					// 空白判定（空白２連はトレーラー部）
					if (str.length() == 0) {

						// ファイルクローズ
						bw.close();
						System.out.println(PBSToolsFileName + " の書き込みが正常に終了しました。");
						System.out.println("---------------------------------------------");

						// ポケモン数リセット
						pokemonDoneNum = 0;

						// ヘッダー部読込前へ
						state = 0;

					// それ以外なら種族名・持ち物名の取得
					} else {

						// ポケモン数が既に6匹に達しているなら読み飛ばし
						if (pokemonDoneNum >= 6) {
							state = 3;
							str = br.readLine();
							continue;
						}

						// ポケモン初期化
						level = 100; //レベル
						speciesName = "けつばん"; // 種族名
						for (int i = 0; i < 4; i++) {
							moveName[i] = "-"; // 技
						}
						itemName = "-"; // 持ち物
						individualValue = "FFFF"; // 個体値
						effortLevel = "63-63-63-63-63"; // 努力レベル

						// 種族名・持ち物名の取得
						sb.delete(0, sb.length());
						int subState = 0;
						for(int i = str.length() - 3; i >= 0; i--) {
							if (subState == 0) {
								if (String.valueOf(str.charAt(i)).equals(")")) {
									sb.delete(0, sb.length());
									subState = 2;
								} else if (String.valueOf(str.charAt(i)).equals("@")) {
									itemName = SD_PBSTools_HelditemNames_Obj.SDToPBSTools(sb.toString().substring(1, sb.length()));
									sb.delete(0, sb.length());
									subState = 1;
								} else {
									sb.insert(0,str.charAt(i));
								}
							} else if (subState == 1 || subState == 2){
								if (subState == 1 && String.valueOf(str.charAt(i)).equals(")")) {
									subState = 2;
								} else if (subState == 2 && String.valueOf(str.charAt(i)).equals("(")) {
									speciesName = SD_PBSTools_SpeciesNames_Obj.SDToPBSTools(sb.toString().substring(0, sb.length() - 1));
									sb.delete(0, sb.length());
									if (!speciesName.equals("けつばん")) {
										i = 0;
									}
									subState = 1;
								} else {
									sb.insert(0,str.charAt(i));
									if (i == 0) {
										speciesName = SD_PBSTools_SpeciesNames_Obj.SDToPBSTools(sb.toString().substring(0, sb.length() - 1));
									}
								}
							}
						}

						// データ部読込中へ
						state = 3;

					}

				// データ部読込中
				} else if (state == 3) {

					// 空白判定（データ部終了）
					if (str.length() == 0) {

						// ポケモン数が既に6匹に達しているなら読み飛ばし
						if (pokemonDoneNum >= 6) {
							state = 2;
							str = br.readLine();
							continue;
						}

						// ファイル書き込み
						sb.delete(0, sb.length());
						sb.append("|");
						sb.append(level);
						sb.append("|");
						sb.append(speciesName);
						sb.append("|");
						sb.append(moveName[0]);
						sb.append("|");
						sb.append("3");
						sb.append("|");
						sb.append(moveName[1]);
						sb.append("|");
						sb.append("3");
						sb.append("|");
						sb.append(moveName[2]);
						sb.append("|");
						sb.append("3");
						sb.append("|");
						sb.append(moveName[3]);
						sb.append("|");
						sb.append("3");
						sb.append("|");
						sb.append(itemName);
						sb.append("|");
						sb.append(individualValue);
						sb.append("|");
						sb.append(effortLevel);
						sb.append("|");
						bw.write(sb.toString());
			        	bw.newLine();

			        	// 技数リセット
			        	moveDoneNum = 0;

			        	// ポケモン数カウントアップ
			        	pokemonDoneNum++;

						// データ部読込前へ
						state = 2;

					// それ以外なら各種情報の取得
					} else {

						// ポケモン数が既に6匹に達しているなら読み飛ばし
						if (pokemonDoneNum >= 6) {
							str = br.readLine();
							continue;
						}

						// レベルの取得
						if (str.substring(0,7).equals("Level: ")) {
							level = Integer.parseInt(str.replace("Level: ", "").replace(" ", ""));
						}

						// 努力レベルの取得
						if (str.substring(0,5).equals("EVs: ")) {
							String HPEffortLevel = " 0";
							String attackEffortLevel = " 0";
							String blockEffortLevel = " 0";
							String speedEffortLevel = " 0";
							String specialEffortLevel = " 0";
							String[] strList = str.replace("EVs: ", "").replace("/ ", "").replace("  ", "").split(" ");
							for (int i = 0; i < strList.length; i = i + 2) {
								if (strList[i + 1].equals("HP")) {
									HPEffortLevel = EVsTranslate(strList[i],level);
								} else if (strList[i + 1].equals("Atk")) {
									attackEffortLevel = EVsTranslate(strList[i],level);
								} else if (strList[i + 1].equals("Def")) {
									blockEffortLevel = EVsTranslate(strList[i],level);
								} else if (strList[i + 1].equals("SpA")) {
									specialEffortLevel = EVsTranslate(strList[i],level);
								} else if (strList[i + 1].equals("Spe")) {
									speedEffortLevel = EVsTranslate(strList[i],level);
								}
							}
							sb.delete(0, sb.length());
							sb.append(HPEffortLevel);
							sb.append("-");
							sb.append(attackEffortLevel);
							sb.append("-");
							sb.append(blockEffortLevel);
							sb.append("-");
							sb.append(speedEffortLevel);
							sb.append("-");
							sb.append(specialEffortLevel);
							effortLevel = sb.toString();
						}

						// 個体値の取得
						if (str.substring(0,5).equals("IVs: ")) {
							String attackIndividualValue = "F";
							String blockIndividualValue = "F";
							String speedIndividualValue = "F";
							String specialIndividualValue = "F";
							String[] strList = str.replace("IVs: ", "").replace("/ ", "").replace("  ", "").split(" ");
							for (int i = 0; i < strList.length; i = i + 2) {
								if (strList[i + 1].equals("Atk")) {
									attackIndividualValue = IVsTranslate(strList[i]);
								} else if (strList[i + 1].equals("Def")) {
									blockIndividualValue = IVsTranslate(strList[i]);
								} else if (strList[i + 1].equals("SpA")) {
									specialIndividualValue = IVsTranslate(strList[i]);
								} else if (strList[i + 1].equals("Spe")) {
									speedIndividualValue = IVsTranslate(strList[i]);
								}
							}
							sb.delete(0, sb.length());
							sb.append(attackIndividualValue);
							sb.append(blockIndividualValue);
							sb.append(speedIndividualValue);
							sb.append(specialIndividualValue);
							individualValue = sb.toString();
						}

						// 技の取得
						if (str.substring(0,2).equals("- ")) {

							// 既に技を４つ読み込んでいるなら読み飛ばし
							if (moveDoneNum >= 4) {
								str = br.readLine();
								continue;
							}

							// 左から順番に技をセット
							moveName[moveDoneNum] = SD_PBSTools_MoveNames_Obj.SDToPBSTools(str.replace("- ", "").replace("  ", ""));
							moveDoneNum++;

						}

					}

				}

				// 次のレコードを読みこむ
    			str = br.readLine();
    		}

		} catch (FileNotFoundException e) {
			System.out.println(SDForderPath + " に " + SDFileName + " という名前のTeamBuilderのテキストファイルが存在しないため処理を終了します。");
		} catch (Exception e) {
			System.out.println(PBSToolsFileName + " の書き込みに失敗しました。");
		} finally {
			try {
				br.close();
			} catch (IOException e) {
			}
		}

	}

	/* 努力レベル変換 */
    public static String EVsTranslate(String inStr, int level) {
    	if (inStr.equals("251") && level <= 50) {
    		return "63";
    	} else {
    		int value = Integer.parseInt(inStr) / 4;
    		if (value >= 0 && value < 10) {
    			return (" " + String.valueOf(value));
    		} else if (value >= 10 && value <= 63) {
    			return String.valueOf(value);
    		} else {
    			return " 0";
    		}
    	}
    }

    /* 個体値変換 */
    public static String IVsTranslate(String inStr) {
    	int value = Integer.parseInt(inStr) / 2;
    	switch (value) {
    	case 0:
    	    return "0";
		case 1:
    	    return "1";
		case 2:
    	    return "2";
		case 3:
    	    return "3";
		case 4:
    	    return "4";
		case 5:
    	    return "5";
		case 6:
    	    return "6";
		case 7:
    	    return "7";
		case 8:
    	    return "8";
		case 9:
    	    return "9";
		case 10:
    	    return "A";
		case 11:
    	    return "B";
		case 12:
    	    return "C";
		case 13:
    	    return "D";
		case 14:
    	    return "E";
		case 15:
    	    return "F";
		default:
    		return "0";
    	}
    }

}
