package main;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Scanner;

import party.PartyInfo;
import validation.EffortLevelCheck;
import validation.IndividualValueCheck;
import validation.ItemClauseCheck;
import validation.LevelLimitationCheckForNC2000;
import validation.LevelValidationCheck;
import validation.MoveBlackListCheck;
import validation.MoveIsolatedCheck;
import validation.MoveSJISSorter;
import validation.MoveWhiteListCheck;
import validation.SpeciesIsolatedCheck;
import validation.SpeciesSJISSorter;
import validation.SpeciesValidationCheckForNC2000;
import validation.ValidationMessageStore;

/*
*  Sample7_ValidationCheckForNC2000クラス
*
*  サンプルのメインクラスです。
*  PBSToolsのパーティテキストを読み込み、ニンテンドウカップ２０００のルールを満たすかどうかを判定します。
*  原則として変換対象のファイル名は対話式で与えますが、第一引数で与えることも可能です。
*  戻り値でエラー数を返します。パーティテキストファイルを読み込めなかった場合は-1を返します。
*
*  変更履歴
*  2021/05/09 仮公開（レベルチェック機能と技チェック機能のみ提供）
*  2021/05/15 完成
*
*/

public class Sample7_ValidationCheckForNC2000 {

	public static void main(String[] args) {

		/* 変数定義 */
		// スキャナー本体
		Scanner scanner = new Scanner(System.in);

		// パス定義
		String fileName = "";
		String PBSToolsForder = "sub_tools" + "\\" + "004_Validation_Check" + "\\" + "Conf";
		String PBSToolsFilePath;
		String dataForder = "sub_tools" + "\\" + "004_Validation_Check" + "\\" + "Data";

		// パーティテキストファイル処理用オブジェクト
		PartyInfo partyInfo = new PartyInfo();

		// Validationチェックのメッセージ格納用オブジェクト
		ValidationMessageStore validationMessageStore = new ValidationMessageStore();

		// Validationチェック用の各種Sorterのオブジェクト
		SpeciesSJISSorter speciesSJISSorter = new SpeciesSJISSorter();
		MoveSJISSorter moveSJISSorter = new MoveSJISSorter();

		// 各種Validationチェック用クラスオブジェクト
		LevelLimitationCheckForNC2000 levelLimitationCheckForNC2000 = new LevelLimitationCheckForNC2000();
		SpeciesIsolatedCheck speciesIsolatedCheck = new SpeciesIsolatedCheck();
		ItemClauseCheck itemClauseCheck = new ItemClauseCheck();
		SpeciesValidationCheckForNC2000 speciesValidationCheckForNC2000 = new SpeciesValidationCheckForNC2000();
		LevelValidationCheck levelValidationCheck = new LevelValidationCheck(dataForder);
		MoveWhiteListCheck moveWhiteListCheck = new MoveWhiteListCheck(dataForder);
		MoveIsolatedCheck moveIsolatedCheck = new MoveIsolatedCheck();
		MoveBlackListCheck moveBlackListCheck = new MoveBlackListCheck(dataForder);
		IndividualValueCheck individualValueCheck = new IndividualValueCheck();
		EffortLevelCheck effortLevelCheck = new EffortLevelCheck();

		/* パーティテキストファイル読み込み */
		// パーティ名入力を求める（第一引数が無い場合）
		if (args.length == 0) {
			System.out.println("\"" + PBSToolsForder + "\\\"にパーティテキストファイルを格納してください。");
			System.out.println("その後、パーティテキストファイル名（拡張子無し）をコンソールに入力して下さい。");
			fileName = scanner.nextLine();
		} else {
			fileName = args[0];
		}
		scanner.close();
		System.out.println("【ファイル名：" + fileName + ".txt】");

		// パス存在確認
		PBSToolsFilePath = PBSToolsForder + "\\" + fileName + ".txt";
		File file = new File(PBSToolsFilePath);
        if (!file.exists()) {
        	System.out.println("パーティテキストファイルが存在しないので処理を終了します。");System.out.println("");
        	System.exit(-1);
        }

        // ファイル読み込み
        BufferedReader br = null;
        try {
        	br = new BufferedReader(new InputStreamReader(new FileInputStream(file),"SJIS"));
        	String str = br.readLine();
        	int i = 0;
        	if(str == null) {
        		throw new NullPointerException();
        	} else {
        		while(str != null && i < 6){
        			partyInfo.partyTextArray[i] = str;
        			i++;
        			str = br.readLine();
        		}
        	}
        } catch (Exception e) {
        	System.out.println("パーティテキストファイルを読み込めないので処理を終了します。");
        } finally {
        	try {
				br.close();
			} catch (IOException e) {
			}
        }
        partyInfo.partyPokeMoldSet();

        /* 全体のバリデーションチェック */

        // ポケモン数・レベル制限チェック（ニンテンドウカップ２０００）
        levelLimitationCheckForNC2000.execute(partyInfo.pokeMoldArray, validationMessageStore);

        // 種族存在・種族重複無しチェック
        speciesIsolatedCheck.execute(partyInfo.pokeMoldArray, validationMessageStore, speciesSJISSorter);

        // 持ち物存在・持ち物重複無しチェック
        itemClauseCheck.execute(partyInfo.pokeMoldArray, validationMessageStore);

        /* 個別のバリデーションチェック */
        int i = 0;
        while(i < 6 && partyInfo.pokeMoldArray[i] != null){

        	// 使用可能種族チェック（ニンテンドウカップ２０００）
        	speciesValidationCheckForNC2000.execute(partyInfo.pokeMoldArray[i], validationMessageStore);

        	// レベル範囲・使用可能レベルチェック
        	levelValidationCheck.execute(partyInfo.pokeMoldArray[i], validationMessageStore);

        	// 覚える技チェック
        	moveWhiteListCheck.execute(partyInfo.pokeMoldArray[i], validationMessageStore, moveSJISSorter);

        	// 技個数・重複無しチェック
        	moveIsolatedCheck.execute(partyInfo.pokeMoldArray[i], validationMessageStore, moveSJISSorter);

        	// 技両立チェック
        	moveBlackListCheck.execute(partyInfo.pokeMoldArray[i], validationMessageStore, moveSJISSorter);

        	// 個体値チェック
        	individualValueCheck.execute(partyInfo.pokeMoldArray[i], validationMessageStore);

        	// 努力レベル可動域チェック
        	effortLevelCheck.execute(partyInfo.pokeMoldArray[i], validationMessageStore);

        	// （次のポケモンの処理へ）
    		i++;
    	}

        /* メッセージ表示 */
        if (validationMessageStore.errorMessagesList.size() > 0) {
        	System.out.println("■エラー");
        	for(String value : validationMessageStore.errorMessagesList) {
        		System.out.println(value);
        	}
        }
        if (validationMessageStore.warningMessagesList.size() > 0) {
        	System.out.println("■ワーニング");
        	for(String value : validationMessageStore.warningMessagesList) {
        		System.out.println(value);
        	}
        }

        /* エラーの数をリターン */
        System.exit(validationMessageStore.errorMessagesList.size());

	}

}
