package validation;

import mold.PokeMold;

/*
*  IndividualValueCheckクラス
*
*  個体値をチェックするクラスです。
*
*  変更履歴
*  2021/05/15 新規作成
*
*/

public class IndividualValueCheck {

	/* 個体値をチェックするメソッド
	 *   引数　：PokeMoldオブジェクト、ValidationMessageStoreオブジェクト
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void execute(PokeMold pokeMold, ValidationMessageStore validationMessageStore) {

    	// 0～15の範囲にあるか確認（想定する使い方ならクリアする）
    	if (pokeMold.HPIndividualValue < 0 || pokeMold.HPIndividualValue > 15 ||
        	pokeMold.attackIndividualValue < 0 || pokeMold.attackIndividualValue > 15 ||
        	pokeMold.blockIndividualValue < 0 || pokeMold.blockIndividualValue > 15 ||
        	pokeMold.speedIndividualValue < 0 || pokeMold.speedIndividualValue > 15 ||
        	pokeMold.specialIndividualValue < 0 || pokeMold.specialIndividualValue > 15) {
        	validationMessageStore.errorMessagesList.add("・" + pokeMold.speciesName + "について、個体値は0～15(F)の範囲で入力して下さい。");
        	return;
        }

    	// NPC交換の個体値制限
    	if (pokeMold.speciesName.equals("イワーク") && pokeMold.level < 5 && (pokeMold.HPIndividualValue != 8 || pokeMold.attackIndividualValue != 9 || pokeMold.blockIndividualValue != 6 || pokeMold.speedIndividualValue != 6 || pokeMold.specialIndividualValue != 6)) {
    		validationMessageStore.errorMessagesList.add("・レベル5未満のイワークの個体値は9666固定です。");
    	} else if (pokeMold.speciesName.equals("ギャロップ") && pokeMold.level < 40 && (pokeMold.HPIndividualValue != 8 || pokeMold.attackIndividualValue != 9 || pokeMold.blockIndividualValue != 6 || pokeMold.speedIndividualValue != 6 || pokeMold.specialIndividualValue != 6)) {
    		validationMessageStore.errorMessagesList.add("・レベル40未満のギャロップの個体値は9666固定です。");
    	} else if (pokeMold.speciesName.equals("ドードリオ") && pokeMold.level < 29 && (pokeMold.HPIndividualValue != 12 || pokeMold.attackIndividualValue != 7 || pokeMold.blockIndividualValue != 7 || pokeMold.speedIndividualValue != 6 || pokeMold.specialIndividualValue != 6)) {
    		validationMessageStore.errorMessagesList.add("・レベル29未満のドードリオの個体値は7766固定です。");
    	} else if (pokeMold.speciesName.equals("ネイティオ") && pokeMold.level < 25 && (pokeMold.HPIndividualValue != 8 || pokeMold.attackIndividualValue != 9 || pokeMold.blockIndividualValue != 6 || pokeMold.speedIndividualValue != 8 || pokeMold.specialIndividualValue != 6)) {
    		validationMessageStore.errorMessagesList.add("・レベル25未満のネイティオの個体値は9686固定です。");
    	} else if (pokeMold.speciesName.equals("ハガネール") && pokeMold.level < 5 && (pokeMold.HPIndividualValue != 8 || pokeMold.attackIndividualValue != 9 || pokeMold.blockIndividualValue != 6 || pokeMold.speedIndividualValue != 6 || pokeMold.specialIndividualValue != 6)) {
    		validationMessageStore.errorMessagesList.add("・レベル5未満のハガネールの個体値は9666固定です。");
    	} else if (pokeMold.speciesName.equals("レアコイル") && pokeMold.level < 30 && (pokeMold.HPIndividualValue != 8 || pokeMold.attackIndividualValue != 9 || pokeMold.blockIndividualValue != 6 || pokeMold.speedIndividualValue != 6 || pokeMold.specialIndividualValue != 6)) {
    		validationMessageStore.errorMessagesList.add("・レベル30未満のレアコイルの個体値は9666固定です。");
    	}

    	return;

    }

}
