package validation;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;

import mold.PokeMold;

/*
*  LevelValidationCheckクラス
*
*  種族毎にレベルが実現可能かをチェックするクラスです。
*
*  変更履歴
*  2021/05/09 新規作成
*
*/

public class LevelValidationCheck {

	// 種族最低レベルCSVファイルから読み込んだデータを保持
	public HashMap<String, Integer> hashMap = new HashMap<String, Integer>();

	// CSVファイル読み込みステータス
	public int CSVFileReadStatus = -1; // -1は未読み込み、0は読込済み、1はファイル未存在、2は読込失敗

	/* デフォルトコンストラクタ：種族最低レベルCSVファイル読み込み
	 */
    public LevelValidationCheck() {
    	readCSV("sub_tools" + "\\" + "004_Validation_Check" + "\\" + "Data");
    }

    /* コンストラクタ：種族最低レベルCSVファイル読み込み（引数でフォルダ指定）
	 */
    public LevelValidationCheck(String dataForder) {
    	readCSV(dataForder);
    }

    /* 種族最低レベルCSVファイル読み込みメソッド
     *   引数　：CSVファイルパス
	 *   戻り値：無し
	 *   例外　：無し
     */
    public void readCSV(String dataForder) {
    	File file = new File(dataForder + "\\levelLowerLimit.csv");
        if (!file.exists()) {
        	CSVFileReadStatus = 1;
        	return;
        }
        BufferedReader br = null;

        try {
        	br = new BufferedReader(new InputStreamReader(new FileInputStream(file),"SJIS"));
        	String str = br.readLine();
        	if(str == null) {
        		throw new NullPointerException();
        	} else {
        		while(str != null){
        			String[] splitStr = str.split("\\,", 0);
        			hashMap.put(splitStr[0], (Integer)Integer.parseInt(splitStr[1]));
        			str = br.readLine();
        		}
        	}
        	CSVFileReadStatus = 0;
        } catch (Exception e) {
        	CSVFileReadStatus = 2;
        } finally {
        	try {
				br.close();
			} catch (IOException e) {
			}
        }
        return;
    }

	/* レベルの妥当性をチェックするメソッド
	 *   引数　：PokeMoldオブジェクト、ValidationMessageStoreオブジェクト
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void execute(PokeMold pokeMold, ValidationMessageStore validationMessageStore) {

    	// チェック対象の種族名、レベルを取得
    	String speciesName = pokeMold.speciesName;
    	int level = pokeMold.level;

    	// レベルが0～255の範囲であることを確認
    	if(level < 0 || level > 255) {
    		validationMessageStore.errorMessagesList.add("・" + speciesName + "のレベルは0～255の範囲にして下さい。");
    		return;
    	}

    	// CSVファイル読み込みステータス判定
    	if (CSVFileReadStatus == -1) {
    		validationMessageStore.warningMessagesList.add("・種族最低レベルCSVファイルが未読込なので" + speciesName + "のレベル妥当性チェックは中断します。レベルが実現可能か別途確認して下さい。");
    		return;
    	} else if (CSVFileReadStatus == 1) {
    		validationMessageStore.warningMessagesList.add("・種族最低レベルCSVファイルが存在しないので" + speciesName + "のレベル妥当性チェックは中断します。レベルが実現可能か別途確認して下さい。");
    		return;
    	} else if (CSVFileReadStatus == 2) {
    		validationMessageStore.warningMessagesList.add("・種族最低レベルCSVファイルの読込に失敗したので" + speciesName + "のレベル妥当性チェックは中断します。レベルが実現可能か別途確認して下さい。");
    		return;
    	}

        // 種族別のレベル妥当性判定
    	Integer lowerLimit = hashMap.get(speciesName);
    	if (lowerLimit == null) {
    		validationMessageStore.warningMessagesList.add("・種族「" + speciesName + "」の実現最低レベルは未定義のため、チェックを飛ばします。");
    		return;
    	}
    	if (lowerLimit > level) {
    		validationMessageStore.errorMessagesList.add("・" + speciesName + "のレベルは" + lowerLimit + "以上にして下さい。");
    	}

    }

}
