package validation;

import mold.PokeMold;

/*
*  MoveIsolatedCheckクラス
*
*  技が０個ではないことや重複がないことをチェックするクラスです。
*  ポイントアップの回数が妥当であることも確認します（想定した方法でPokeMoldオブジェクトが生成されているならOKになります）。
*
*  変更履歴
*  2021/05/09 新規作成
*
*/

public class MoveIsolatedCheck {

	/* 技の妥当性をチェックするメソッド
	 *   引数　：PokeMoldオブジェクト、ValidationMessageStoreオブジェクト、MoveSJISSorterオブジェクト
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void execute(PokeMold pokeMold, ValidationMessageStore validationMessageStore, MoveSJISSorter moveSJISSorter) {

    	// 技が０個ではないことや重複がないことをチェック
    	String[] moveNameSorted = new String[4];
    	moveNameSorted = moveSJISSorter.execute(pokeMold.moveName);
    	int i = 0;
    	if (moveNameSorted[i] == null) {
    		validationMessageStore.errorMessagesList.add("・" + pokeMold.speciesName + "に技を１つ以上持たせて下さい。");
    		return;
    	}
    	i++;
    	for (; i < 4; i++) {
    		if (moveNameSorted[i] != null) {
        		if (moveNameSorted[i].equals(moveNameSorted[i-1])) {
        			validationMessageStore.errorMessagesList.add("・" + pokeMold.speciesName + "の" + moveNameSorted[i] + "の技が重複しています。");
        		}
        	}
    	}

    	// ポイントアップの回数が妥当であることのチェック
    	for (i = 0; i < 4; i++) {
    		if (moveSJISSorter.comparator.getSortOrder(pokeMold.moveName[i]) != 65535) {
    			if (pokeMold.movePointUpTimes[i] < 0 || pokeMold.movePointUpTimes[i] > 3) {
    				validationMessageStore.errorMessagesList.add("・" + pokeMold.speciesName + "の" + pokeMold.moveName[i] + "のポイントアップ回数は0～3にして下さい。");
    			}
    		}
    	}

    	return;

    }

}
