package validation;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import mold.PokeMold;

/*
*  MoveWhiteListCheckクラス
*
*  各種族について、覚える技の判定を行います。
*
*  変更履歴
*  2021/05/09 新規作成
*
*/

public class MoveWhiteListCheck {

	// ホワイトリストCSVファイルから読み込んだデータを保持
	private String[] dataRecord = new String[3]; // 読み込んだ最新のレコード
	public List<String[]> dataList = new ArrayList<String[]>();

	// CSVファイル読み込みステータス
	public int CSVFileReadStatus = -1; // -1は未読み込み、0は読込済み、1はファイル未存在、2は読込失敗

	/* デフォルトコンストラクタ：種族最低レベルCSVファイル読み込み
	 */
    public MoveWhiteListCheck() {
    	readCSV("sub_tools" + "\\" + "004_Validation_Check" + "\\" + "Data");
    }

    /* コンストラクタ：ホワイトリストCSVファイル読み込み（引数でフォルダ指定）
	 */
    public MoveWhiteListCheck(String dataForder) {
    	readCSV(dataForder);
    }

    /* 種族最低レベルCSVファイル読み込みメソッド
     *   引数　：CSVファイルパス
	 *   戻り値：無し
	 *   例外　：無し
     */
    public void readCSV(String dataForder) {
    	File file = new File(dataForder + "\\moveWhiteList.csv");
        if (!file.exists()) {
        	CSVFileReadStatus = 1;
        	return;
        }
        BufferedReader br = null;

        try {
        	br = new BufferedReader(new InputStreamReader(new FileInputStream(file),"SJIS"));
        	String str = br.readLine();
        	if(str == null) {
        		throw new NullPointerException();
        	} else {
        		while(str != null){
        			dataRecord = str.split("\\,", 0);
        			dataList.add(dataRecord);
        			str = br.readLine();
        		}
        	}
        	CSVFileReadStatus = 0;
        } catch (Exception e) {
        	CSVFileReadStatus = 2;
        } finally {
        	try {
				br.close();
			} catch (IOException e) {
			}
        }
        return;
    }

	/* 技のホワイトリストチェックするメソッド
	 *   引数　：PokeMoldオブジェクト、ValidationMessageStoreオブジェクト、MoveSJISSorterオブジェクト
	 *   戻り値：無し
	 *   例外　：無し
	 */
    public void execute(PokeMold pokeMold, ValidationMessageStore validationMessageStore, MoveSJISSorter moveSJISSorter) {

    	// メモリに格納したCSVファイルから当該種族のデータのみ抜き出し
    	int state = 0;
    	List<String[]> dataListSpecies = new ArrayList<String[]>();
    	for (String[] currentRecord : dataList) {
    		if (state == 0 && pokeMold.speciesName.equals(currentRecord[0])) {
    			state = 1;
    			dataListSpecies.add(currentRecord);
    			continue;
    		} else if (state == 1 && pokeMold.speciesName.equals(currentRecord[0])) {
    			dataListSpecies.add(currentRecord);
    		} else if (state == 1 && !pokeMold.speciesName.equals(currentRecord[0])) {
    			state = 2;
    			break;
    		}
    	}

    	// 当該種族のデータを用いてホワイトリストチェックを実施
    	for (int i = 0; i < 4; i++) {
    		if (moveSJISSorter.comparator.getSortOrder(pokeMold.moveName[i]) != 65535) {
    			boolean isFound = false;
    			int lowerLimitLevel = 0;
    			for (String[] currentRecord : dataListSpecies) {
    				if (pokeMold.moveName[i].equals(currentRecord[1])) {
    					isFound = true;
    					lowerLimitLevel = Integer.parseInt(currentRecord[2]);
    					break;
    				}
    			}
    			if (!isFound) {
    				validationMessageStore.errorMessagesList.add("・" + pokeMold.speciesName + "は" + pokeMold.moveName[i] + "を覚えません。");
    			} else if (pokeMold.level < lowerLimitLevel) {
    				validationMessageStore.errorMessagesList.add("・" + pokeMold.speciesName + "は" + pokeMold.moveName[i] + "をレベル" + lowerLimitLevel + "以上でないと覚えません。");
    			}
    		}
    	}

    }

}
