package validation;

import java.util.Comparator;
import java.util.HashMap;

/*
*  SpeciesSJISSorterComparatorクラス
*
*  種族配列のSJIS順のソート順を定義します。
*  Validation向けのクラスです。
*
*  変更履歴
*  2021/05/09 新規作成
*  2021/05/16 ニドラン♂とニドラン♀のソート順が逆であったのを修正
*
*/

public class SpeciesSJISSorterComparator implements Comparator<String> {

	// 種族のソート順を格納するHashMap
	public HashMap<String, Integer> speciesMap = new HashMap<String, Integer>();

	/* デフォルトコンストラクタ：ソート順作成
	 */
    public SpeciesSJISSorterComparator() {
    	speciesMap.put("アーボ",1);
    	speciesMap.put("アーボック",2);
    	speciesMap.put("アズマオウ",3);
    	speciesMap.put("アリアドス",4);
    	speciesMap.put("アリゲイツ",5);
    	speciesMap.put("アンノーン",6);
    	speciesMap.put("イーブイ",7);
    	speciesMap.put("イシツブテ",8);
    	speciesMap.put("イトマル",9);
    	speciesMap.put("イノムー",10);
    	speciesMap.put("イワーク",11);
    	speciesMap.put("ウインディ",12);
    	speciesMap.put("ウソッキー",13);
    	speciesMap.put("ウツドン",14);
    	speciesMap.put("ウツボット",15);
    	speciesMap.put("ウパー",16);
    	speciesMap.put("ウリムー",17);
    	speciesMap.put("エーフィ",18);
    	speciesMap.put("エアームド",19);
    	speciesMap.put("エイパム",20);
    	speciesMap.put("エビワラー",21);
    	speciesMap.put("エレキッド",22);
    	speciesMap.put("エレブー",23);
    	speciesMap.put("エンテイ",24);
    	speciesMap.put("オーダイル",25);
    	speciesMap.put("オオタチ",26);
    	speciesMap.put("オクタン",27);
    	speciesMap.put("オコリザル",28);
    	speciesMap.put("オタチ",29);
    	speciesMap.put("オドシシ",30);
    	speciesMap.put("オニスズメ",31);
    	speciesMap.put("オニドリル",32);
    	speciesMap.put("オムスター",33);
    	speciesMap.put("オムナイト",34);
    	speciesMap.put("カイリキー",35);
    	speciesMap.put("カイリュー",36);
    	speciesMap.put("カイロス",37);
    	speciesMap.put("カビゴン",38);
    	speciesMap.put("カブト",39);
    	speciesMap.put("カブトプス",40);
    	speciesMap.put("カポエラー",41);
    	speciesMap.put("カメール",42);
    	speciesMap.put("カメックス",43);
    	speciesMap.put("カモネギ",44);
    	speciesMap.put("カラカラ",45);
    	speciesMap.put("ガーディ",46);
    	speciesMap.put("ガラガラ",47);
    	speciesMap.put("ガルーラ",48);
    	speciesMap.put("キマワリ",49);
    	speciesMap.put("キャタピー",50);
    	speciesMap.put("キュウコン",51);
    	speciesMap.put("キリンリキ",52);
    	speciesMap.put("キレイハナ",53);
    	speciesMap.put("キングドラ",54);
    	speciesMap.put("キングラー",55);
    	speciesMap.put("ギャラドス",56);
    	speciesMap.put("ギャロップ",57);
    	speciesMap.put("クサイハナ",58);
    	speciesMap.put("クヌギダマ",59);
    	speciesMap.put("クラブ",60);
    	speciesMap.put("クロバット",61);
    	speciesMap.put("グライガー",62);
    	speciesMap.put("グランブル",63);
    	speciesMap.put("ケーシィ",64);
    	speciesMap.put("ケンタロス",65);
    	speciesMap.put("ゲンガー",66);
    	speciesMap.put("コイキング",67);
    	speciesMap.put("コイル",68);
    	speciesMap.put("コクーン",69);
    	speciesMap.put("コダック",70);
    	speciesMap.put("コラッタ",71);
    	speciesMap.put("コンパン",72);
    	speciesMap.put("ゴース",73);
    	speciesMap.put("ゴースト",74);
    	speciesMap.put("ゴーリキー",75);
    	speciesMap.put("ゴマゾウ",76);
    	speciesMap.put("ゴルダック",77);
    	speciesMap.put("ゴルバット",78);
    	speciesMap.put("ゴローニャ",79);
    	speciesMap.put("ゴローン",80);
    	speciesMap.put("サイドン",81);
    	speciesMap.put("サイホーン",82);
    	speciesMap.put("サナギラス",83);
    	speciesMap.put("サニーゴ",84);
    	speciesMap.put("サワムラー",85);
    	speciesMap.put("サンダー",86);
    	speciesMap.put("サンダース",87);
    	speciesMap.put("サンド",88);
    	speciesMap.put("サンドパン",89);
    	speciesMap.put("シードラ",90);
    	speciesMap.put("シェルダー",91);
    	speciesMap.put("シャワーズ",92);
    	speciesMap.put("ジュゴン",93);
    	speciesMap.put("スイクン",94);
    	speciesMap.put("スターミー",95);
    	speciesMap.put("ストライク",96);
    	speciesMap.put("スピアー",97);
    	speciesMap.put("スリーパー",98);
    	speciesMap.put("スリープ",99);
    	speciesMap.put("ズバット",100);
    	speciesMap.put("セレビィ",101);
    	speciesMap.put("ゼニガメ",102);
    	speciesMap.put("ソーナンス",103);
    	speciesMap.put("タッツー",104);
    	speciesMap.put("タマタマ",105);
    	speciesMap.put("ダグトリオ",106);
    	speciesMap.put("チコリータ",107);
    	speciesMap.put("チョンチー",108);
    	speciesMap.put("ツボツボ",109);
    	speciesMap.put("テッポウオ",110);
    	speciesMap.put("ディグダ",111);
    	speciesMap.put("デリバード",112);
    	speciesMap.put("デルビル",113);
    	speciesMap.put("デンリュウ",114);
    	speciesMap.put("トゲチック",115);
    	speciesMap.put("トゲピー",116);
    	speciesMap.put("トサキント",117);
    	speciesMap.put("トランセル",118);
    	speciesMap.put("ドードリオ",119);
    	speciesMap.put("ドードー",120);
    	speciesMap.put("ドーブル",121);
    	speciesMap.put("ドガース",122);
    	speciesMap.put("ドククラゲ",123);
    	speciesMap.put("ドンファン",124);
    	speciesMap.put("ナゾノクサ",125);
    	speciesMap.put("ナッシー",126);
    	speciesMap.put("ニドキング",127);
    	speciesMap.put("ニドクイン",128);
    	speciesMap.put("ニドラン♂",129);
    	speciesMap.put("ニドラン♀",130);
    	speciesMap.put("ニドリーナ",131);
    	speciesMap.put("ニドリーノ",132);
    	speciesMap.put("ニャース",133);
    	speciesMap.put("ニューラ",134);
    	speciesMap.put("ニョロゾ",135);
    	speciesMap.put("ニョロトノ",136);
    	speciesMap.put("ニョロボン",137);
    	speciesMap.put("ニョロモ",138);
    	speciesMap.put("ヌオー",139);
    	speciesMap.put("ネイティ",140);
    	speciesMap.put("ネイティオ",141);
    	speciesMap.put("ノコッチ",142);
    	speciesMap.put("ハガネール",143);
    	speciesMap.put("ハクリュー",144);
    	speciesMap.put("ハッサム",145);
    	speciesMap.put("ハネッコ",146);
    	speciesMap.put("ハピナス",147);
    	speciesMap.put("ハリーセン",148);
    	speciesMap.put("バクフーン",149);
    	speciesMap.put("バタフリー",150);
    	speciesMap.put("バリヤード",151);
    	speciesMap.put("バルキー",152);
    	speciesMap.put("バンギラス",153);
    	speciesMap.put("パウワウ",154);
    	speciesMap.put("パラス",155);
    	speciesMap.put("パラセクト",156);
    	speciesMap.put("パルシェン",157);
    	speciesMap.put("ヒトカゲ",158);
    	speciesMap.put("ヒトデマン",159);
    	speciesMap.put("ヒノアラシ",160);
    	speciesMap.put("ヒマナッツ",161);
    	speciesMap.put("ヒメグマ",162);
    	speciesMap.put("ビードル",163);
    	speciesMap.put("ビリリダマ",164);
    	speciesMap.put("ピィ",165);
    	speciesMap.put("ピカチュウ",166);
    	speciesMap.put("ピクシー",167);
    	speciesMap.put("ピジョット",168);
    	speciesMap.put("ピジョン",169);
    	speciesMap.put("ピチュー",170);
    	speciesMap.put("ピッピ",171);
    	speciesMap.put("フーディン",172);
    	speciesMap.put("ファイヤー",173);
    	speciesMap.put("フォレトス",174);
    	speciesMap.put("フシギソウ",175);
    	speciesMap.put("フシギダネ",176);
    	speciesMap.put("フシギバナ",177);
    	speciesMap.put("フリーザー",178);
    	speciesMap.put("ブースター",179);
    	speciesMap.put("ブーバー",180);
    	speciesMap.put("ブビィ",181);
    	speciesMap.put("ブラッキー",182);
    	speciesMap.put("ブルー",183);
    	speciesMap.put("プクリン",184);
    	speciesMap.put("プテラ",185);
    	speciesMap.put("ププリン",186);
    	speciesMap.put("プリン",187);
    	speciesMap.put("ヘラクロス",188);
    	speciesMap.put("ヘルガー",189);
    	speciesMap.put("ベイリーフ",190);
    	speciesMap.put("ベトベター",191);
    	speciesMap.put("ベトベトン",192);
    	speciesMap.put("ベロリンガ",193);
    	speciesMap.put("ペルシアン",194);
    	speciesMap.put("ホーホー",195);
    	speciesMap.put("ホウオウ",196);
    	speciesMap.put("ポッポ",197);
    	speciesMap.put("ポニータ",198);
    	speciesMap.put("ポポッコ",199);
    	speciesMap.put("ポリゴン",200);
    	speciesMap.put("ポリゴン２",201);
    	speciesMap.put("マグカルゴ",202);
    	speciesMap.put("マグマッグ",203);
    	speciesMap.put("マグマラシ",204);
    	speciesMap.put("マタドガス",205);
    	speciesMap.put("マダツボミ",206);
    	speciesMap.put("マリル",207);
    	speciesMap.put("マリルリ",208);
    	speciesMap.put("マルマイン",209);
    	speciesMap.put("マンキー",210);
    	speciesMap.put("マンタイン",211);
    	speciesMap.put("ミニリュウ",212);
    	speciesMap.put("ミュウ",213);
    	speciesMap.put("ミュウツー",214);
    	speciesMap.put("ミルタンク",215);
    	speciesMap.put("ムウマ",216);
    	speciesMap.put("ムチュール",217);
    	speciesMap.put("メガニウム",218);
    	speciesMap.put("メタモン",219);
    	speciesMap.put("メノクラゲ",220);
    	speciesMap.put("メリープ",221);
    	speciesMap.put("モココ",222);
    	speciesMap.put("モルフォン",223);
    	speciesMap.put("モンジャラ",224);
    	speciesMap.put("ヤドキング",225);
    	speciesMap.put("ヤドラン",226);
    	speciesMap.put("ヤドン",227);
    	speciesMap.put("ヤミカラス",228);
    	speciesMap.put("ヤンヤンマ",229);
    	speciesMap.put("ユンゲラー",230);
    	speciesMap.put("ヨーギラス",231);
    	speciesMap.put("ヨルノズク",232);
    	speciesMap.put("ライコウ",233);
    	speciesMap.put("ライチュウ",234);
    	speciesMap.put("ラッキー",235);
    	speciesMap.put("ラッタ",236);
    	speciesMap.put("ラフレシア",237);
    	speciesMap.put("ラプラス",238);
    	speciesMap.put("ランターン",239);
    	speciesMap.put("リザード",240);
    	speciesMap.put("リザードン",241);
    	speciesMap.put("リングマ",242);
    	speciesMap.put("ルージュラ",243);
    	speciesMap.put("ルギア",244);
    	speciesMap.put("レアコイル",245);
    	speciesMap.put("レディアン",246);
    	speciesMap.put("レディバ",247);
    	speciesMap.put("ロコン",248);
    	speciesMap.put("ワタッコ",249);
    	speciesMap.put("ワニノコ",250);
    	speciesMap.put("ワンリキー",251);
    }

	/* ソート順を返すメソッド
	 *   引数　：種族名（String）
	 *   戻り値：ソート順（int）
	 *   例外　：無し
	 */
    public int getSortOrder(String speciesName) {
    	Integer speciesSortOrder = speciesMap.get(speciesName);
    	if (speciesSortOrder == null) {
    		speciesSortOrder = 65535;
    	}
    	return speciesSortOrder;
    }

	/* compareメソッド
	 */
    @Override
    public int compare(String o1, String o2) {
        return getSortOrder(o1) - getSortOrder(o2);
    }

}
